@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -56,6 +56,7 @@
 -type elementary_option() ::
         {active, true | false | once | -32768..32767} |
         {buffer, non_neg_integer()} |
+        {non_block_send, boolean()} |
         {debug, boolean()} |
         {dontroute, boolean()} |
         {exclusiveaddruse, boolean()} |
@@ -85,6 +86,7 @@
 -type elementary_option_name() ::
         active |
         buffer |
+        non_block_send |
         debug |
         dontroute |
         exclusiveaddruse |
